from .Param import Param

class Part:
    def __init__(self, part) -> None:
        self.part = part
    
    @property
    def name(self) -> str:
        return self.part.name
    
    @property
    def model(self) -> str:
        return self.part.model
    
    @property
    def parameters(self) -> dict[str, Param]:
        return {k:Param(x) for (k,x) in self.part.parameters.items()}



import os
import sys
import psutil
import inspect
from .Workspace import Workspace

class Application:
    """
    A class for Application
    ...

    Attributes
    ----------
    current_workspace : str
        workspace object of active workspace
    systemvue_path : str
        path to SystemVue bin directory
    is_internal : bool
        boolean of if application is using SystemVue internal or external

    Methods
    -------
    open_workspace(path : str) -> Workspace
        opens workspace with path and returns workspace object
    create_workspace() -> Workspace
        creates workspace and returns it
    show(value)
        shows value in output window in SystemVue
    """

    # static variables managing initialization
    # __is_initialized = False
    __is_internal = False
    _SystemVue_path = ""
    __externalModuleName = "SystemVue_External.pyd"

    def __init__(self) -> None:
        if sys.version_info[0] != 3 or sys.version_info[1] != 10:
            raise Exception("Need Python 3.10")
        # if not Application.__is_initialized:
        process = psutil.Process(os.getpid())
        if "SystemVue" in process.name() or "Genesys" in process.name():
            Application.__is_internal = True
            import SystemVue_Internal as systemvueapi
            # Application.__is_initialized = True
        
        else:
            # we need a SystemVue to work with. Try the following in order:
            # check if the user has called set_SystemVue_path
            # check if we're in a SystemVue install area by checking if the pyd module is nearby
            # check for a SV install in the registry 
            # fail

             # check static var from set_SystemVue_path
            if Application._SystemVue_path != "":
                if not os.path.exists( os.path.join(Application._SystemVue_path, Application.__externalModuleName )):
                    raise RuntimeError(f"SystemVue path has been set to {Application._SystemVue_path}, but module was not found at this location.")

            else:
                sourceFileDir = os.path.dirname(inspect.getsourcefile(lambda:0))
                systemVueInstallBin = os.path.abspath(os.path.join(sourceFileDir, "../../bin"))
        
                if os.path.exists( os.path.join(systemVueInstallBin, Application.__externalModuleName )):
                    Application._SystemVue_path = systemVueInstallBin

                else:
                    # try the registry
                    import winreg
                    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, "SOFTWARE\\Keysight\\SystemVue2023\\System", access=winreg.KEY_READ | winreg.KEY_WOW64_64KEY)
                    path = winreg.QueryValueEx(key, "DefaultPath")
                    if os.path.exists(path[0]):
                        Application._SystemVue_path = path[0] + "Bin"
                    else:
                        # raise exception - can't figure it out
                        raise RuntimeError('Cannot find SystemVue_External.pyd. Make sure the path to the SystemVue bin folder is added to sys.path.') 

            sys.path.insert(0, Application._SystemVue_path)
        
            # do a try catch here and explain in the exception *all* the places we looked for the module
            try:
                import SystemVue_External as systemvueapi
                # Application.__is_initialized = True
            except Exception as e:
                print(e)
                exit

        self.app = systemvueapi.Application()
    
    def open_workspace(self, path: str) -> Workspace:
        """
        Opens workspace with path and returns workspace object

        Parameters
        -----------
        path : str
            path to workspace to open
        """
        return Workspace(self.app.open_workspace(path))

    def create_workspace(self) -> Workspace:
        """
        Creates a workspace and returns it
        """
        return Workspace(self.app.create_workspace())
    
    def exit(self):
        """
        Exits the application
        """
        self.app.exit()

    def show(self, value):
        """
        Shows value in output window in SystemVue

        Parameters
        -----------
        value
            value to be shown in output window
        """
        return self.app.show(value)
    
    @property
    def current_workspace(self) -> Workspace:
        """
        Returns workspace object of active workspace
        """
        return Workspace(self.app.current_workspace)

    # Document that this only takes effect if set before the first call to Application()
    # Add runtime error if set after initialization has been done

    @property
    def systemvue_path(self) -> Workspace:
        return Application.__SystemVue_path

    @systemvue_path.setter
    def systemvue_path(self, path):
        Application.__SystemVue_path = path

    # add property: is_internal



class Equations:
    def __init__(self, equation) -> None:
        self.equation = equation

    def run(self):
        self.equation.run()
    
    @property
    def name(self) -> str:
        return self.equation.name
    
    @property
    def text(self) -> str:
        return self.equation.text

    @text.setter
    def text(self, eqnText: str):
        self.equation.text = eqnText



from enum import Enum

class Return(Enum):
    ValueOnly = 0
    IndepsAndValue = 1
    IndepValue = 2

class SingleValueIndeps(Enum):
    Include = 0
    Exclude = 1

class RequireAllPointsExist(Enum):
    No = 0
    #Yes = 1

_SliceDataOptionsOrder = [Return, SingleValueIndeps, RequireAllPointsExist]
_SliceDataOptionsDefaults = {
    Return: Return.IndepsAndValue.value, 
    SingleValueIndeps: SingleValueIndeps.Exclude.value, 
    RequireAllPointsExist: RequireAllPointsExist.No.value
}



from enum import Enum

class Return(Enum):
    ValueOnly = 0
    IndepsAndValue = 1
    IndepValue = 2

class SingleValueIndeps(Enum):
    Include = 0
    Exclude = 1

class Interpolate(Enum):
    Linear = 0
    InterpolateSpline = 1
    InterpolateCubic = 2

class InterpolateDomain(Enum):
    Rectangular = 0
    Polar = 1
    dB = 2

class Extrapolate(Enum):
    No = 0
    Zero = 1
    Constant = 2
    Interpolate = 3

class FreeVariables(Enum):
    All = 0
    #NearbyUnion = 1
    #NearbyIntersection = 2
    #NearbyNearest = 3

_InterpDataOptionsOrder = [Return, SingleValueIndeps, Interpolate, InterpolateDomain, Extrapolate, FreeVariables]
_InterpDataOptionsDefaults = {
    Return: Return.IndepsAndValue.value,
    SingleValueIndeps: SingleValueIndeps.Exclude.value,
    Interpolate: Interpolate.Linear.value,
    InterpolateDomain: InterpolateDomain.Rectangular.value,
    Extrapolate: Extrapolate.Constant.value,
    FreeVariables: FreeVariables.All.value
}



from .Analysis import Analysis
from .DataSet import DataSet
from .Design import Design
from .Equations import Equations
from .Graph import Graph
from .Variable import Variable

class Workspace:
    """
    A class for Workspace
    ...

    Attributes
    ----------
    workspace_path : str
        path to workspace
    name : str
        name of workspace
    datasets : dict[str, DataSet]
        dict of (name, dataset) as the (key, value) pair in the workspace
    analyses : dict[str, Analysis]
        dict of (name, analysis) as the (key, value) pair in the workspace
    equations : dict[str, Equations]
        dict of (name, equations) as the (key, value) pair in the workspace
    graphs : dict[str, Graph]
        dict of (name, graph) as the (key, value) pair in the workspace
    variables : dict[str, Variable]
        dict of (name, variable) as the (key, value) pair in the workspace
    designs : dict[str, Design]
        dict of (name, design) as the (key, value) pair in the workspace

    Methods
    -------
    create_dataset -> DataSet
        creates a dataset and returns it
    addVariable() -> Variable
        adds variable to workspace and returns the variable
    """
    def __init__(self, ws) -> None:
        self.ws = ws

    def create_dataset(self, dataSetName: str) -> DataSet:
        """
        Creates a dataset and returns it
        """
        return DataSet(self.ws.create_dataset(dataSetName))

    def add_variable(self, varName:str) -> Variable:
        """
        Adds variable to workspace and returns the variable
        """
        return self.ws.add_variable(varName)

    def save(self) -> None:
        """
        Save the workspace
        """
        self.ws.save()

    @property
    def name(self) -> str:
        """
        Returns name of workspace
        """
        return self.ws.name

    @property
    def workspace_path(self) -> str:
        """
        Returns path to workspace
        """
        return self.ws.workspace_path

    @property
    def analyses(self) -> dict[str, Analysis]:
        """
        Returns dict of (name, analysis) as the (key, value) pair in the workspace
        """
        return {k:Analysis(x) for (k,x) in self.ws.analyses.items()}

    @property
    def datasets(self) -> dict[str, DataSet]:
        """
        Returns dict of (name, dataset) as the (key, value) pair in the workspace
        """
        return {k:DataSet(x) for (k,x) in self.ws.datasets.items()}

    @property
    def designs(self) -> dict[str, Design]:
        """
        dict of (name, design) as the (key, value) pair in the workspace
        """
        return {k:Design(x) for (k,x) in self.ws.designs.items()}

    @property
    def equations(self) -> dict[str, Equations]:
        """
        Returns dict of (name, equations) as the (key, value) pair in the workspace
        """
        return {k:Equations(x) for (k,x) in self.ws.equations.items()}

    @property
    def graphs(self) -> dict[str, Graph]:
        """
        Returns dict of (name, graph) as the (key, value) pair in the workspace
        """
        return {k:Graph(x) for (k,x) in self.ws.graphs.items()}

    @property
    def variables(self) -> dict[str, Variable]:
        """
        dict of (name, variable) as the (key, value) pair in the workspace
        """
        return {k:Variable(x) for (k,x) in self.ws.variables.items()}



class Param:
    def __init__(self, param) -> None:
        self.param = param
    
    @property
    def name(self) -> str:
        return self.param.name
    
    @property
    def units(self) -> str:
        return self.param.units
    
    @property
    def value(self):
        return self.param.value
    
    @value.setter
    def value(self, val):
        self.param.value = val



from __future__ import annotations
import pandas

from . import SlicingOptions
from . import InterpOptions

class Variable:
    def __init__(self, variable) -> None:
        self.variable = variable

    @property
    def name(self) -> str:
        return self.variable.name
    
    @property
    def units(self) -> str:
        return self.variable.units
    
    @property
    def indeps(self) -> Sequence[Variable]:
        return [Variable(x) for (x) in self.variable.indeps]
    
    @indeps.setter
    def indeps(self, indeps):
        self.variable.indeps = [x.variable for (x) in indeps]

    @property
    def value(self):
        return self.variable.value

    @value.setter
    def value(self, val):
        self.variable.value = val

    @property
    def dataframe(self) -> pandas.DataFrame:
        df = self.variable.dataframe
        overall = {}
        indeps = []
        deps = []
        for var in df[0]:
            indeps.append(var.value)
            name = var.name + " (Indep)"
            overall[name] = var.value
        for var in df[1]:
            deps.append(var.value)
            name = var.name + " (Dep)"
            overall[name] = var.value
        return pandas.DataFrame(overall)

    @staticmethod
    def _build_options_map(options_list: list):
        """
        Convert a list of options (enum types) to a map with exactly one value for each type
        """
        opts = {}
        if options_list is None:
            return opts

        for option in options_list:
            if type(option) in opts and opts[type(option)] != option.value:
                raise RuntimeError("Cannot specify contradictory option values")

            opts[type(option)] = option.value 

        return opts

    @staticmethod
    def __get_slice_options(options_list):
        final_options = SlicingOptions._SliceDataOptionsDefaults.copy()
        final_options.update(Variable._build_options_map(options_list))
        return [final_options[feature] for feature in SlicingOptions._SliceDataOptionsOrder]

    @staticmethod
    def __get_interp_options(options_list):
        final_options = InterpOptions._InterpDataOptionsDefaults.copy()
        final_options.update(Variable._build_options_map(options_list))
        return [final_options[feature] for feature in InterpOptions._InterpDataOptionsOrder]    

    @staticmethod
    def __unpack_tuples(zipped_list):
        """
        Unpack a list of 2-tuples into two lists---one of the first tuple elements and one of the second.

        This function is the inverse of 'zip'. Given a list of 2-tuples, return 2 lists where the first
        contains all of the first tuple elements and the second list contains all of the second elements.
        Example:
        input of the form [(A, 1), (B, 2), ..., (Z,26)]
        output of the form ([A, B, ..., Z], [1, 2, ..., 26])
        """
        if zipped_list is None or len(zipped_list) == 0:
            return [(), ()]

        return list(zip(*zipped_list))

    def slice_data(self, query_items = None, slice_options: list = None) -> Variable:
        """
        Get a subset of data from a variable according to indep params, following certain slicing options.

        query_items has the following form:
        [ (indep_name1: indep_value1), (indep_name2: indep_value2), ... , (indep_nameN: indep_valueN) ]
        where name is the name of an indep and value is the associated search value for that indep
        
        slice_options is a list of the enumerated options
        [ option1, option2, ..., optionN ]
        if user specifies multiple contradictory values for an option we will throw an exception
        """
        indep_names, indep_values = Variable.__unpack_tuples(query_items)
        return Variable(self.variable.slice_data(indep_names, indep_values, Variable.__get_slice_options(slice_options)))

    def interp_data(self, query_items = None, interp_options: list = None) -> Variable:
        """
        Get a interpolated data from a variable according to indep params according to interpolation options.

        query_items has the following form:
        [ (indep_name1: indep_value1), (indep_name2: indep_value2), ... , (indep_nameN: indep_valueN) ]
        where name is the name of an indep and value is the associated search value for that indep
        
        interp_options is a list of the enumerated options
        [ option1, option2, ..., optionN ]
        if user specifies multiple contradictory values for an option we will throw an exception
        """
        indep_names, indep_values = Variable.__unpack_tuples(query_items)
        return Variable(self.variable.interp_data(indep_names, indep_values, Variable.__get_interp_options(interp_options)))

    @property
    def entry(self):
        return self.variable.entry
    
    @entry.setter
    def entry(self, e):
        self.variable.entry= e




from .DataSet import DataSet
from .Design import Design


class Analysis:
    def __init__(self, analysis) -> None:
        self.analysis = analysis

    def run(self):
        self.analysis.run()
    
    @property
    def name(self) -> str:
        return self.analysis.name
    
    @property
    def dataset_name(self) -> str:
        return self.analysis.dataset_name

    @dataset_name.setter
    def dataset_name(self, name: str):
        self.analysis.dataset_name = name
    
    @property
    def design_name(self) -> str:
        return self.analysis.design_name

    @design_name.setter
    def design_name(self, name: str):
        self.analysis.design_name = name
    
    @property
    def design(self) -> Design:
        return Design(self.analysis.design)
    
    @property
    def dataset(self) -> DataSet:
        return DataSet(self.analysis.dataset)



from .Variable import Variable

class DataSet:
    def __init__(self, dataset) -> None:
        self.dataset = dataset

    def create_variable(self, name: str) -> Variable:
        return Variable(self.dataset.create_variable(name))

    def delete_variable(self, name: str):
        self.dataset.delete_variable(name)

    def import_data(self, fileSpecification: str, format: str, options):
        self.dataset.import_data(fileSpecification, format, options)

    def export_data(self, fileSpecification: str, format: str, options):
        self.dataset.export_data(fileSpecification, format, options)
    
    @property
    def name(self) -> str:
        return self.dataset.name
    
    @property
    def variables(self) -> dict[str, Variable]:
        return {k:Variable(x) for (k,x) in self.dataset.variables.items()}



from .Param import Param
from .Part import Part

class Design:
    def __init__(self, design) -> None:
        self.design = design
    
    @property
    def name(self) -> str:
        return self.design.name
    
    @property
    def parts(self) -> dict[str, Part]:
        return {k:Part(x) for (k,x) in self.design.parts.items()}
    
    @property
    def parameters(self) -> dict[str, Param]:
        return {k:Param(x) for (k,x) in self.design.parameters.items()}



import setuptools

setuptools.setup(

    name="keysight-systemvue", # Replace with your own username

    description="Python package for SystemVue 2023",

    version="2023.1",

    include_package_data=True,

    package_data={'keysight/systemvue': ['*']},

    packages=setuptools.find_namespace_packages(include=['keysight.*']),

    classifiers=[

        "Programming Language :: Python :: 3",

    ],

    install_requires=["numpy", "pandas", "psutil", "matplotlib"],

    python_requires='==3.10.*',

)


from .GraphSeries import GraphSeries

class Graph:
    def __init__(self, graph) -> None:
        self.graph = graph

    def export_graph_as_image(self, path: str, format: str, width: int, height: int, dpi: int):
        self.graph.export_graph_as_image(path, format, width, height, dpi)

    def set_dataset_reference(self, series: str, dataset: str):
        self.graph.set_dataset_reference(series, dataset)

    def set_dataset_variable(self, series: str, variable: str):
        self.graph.set_dataset_variable(series, variable)

    def open(self):
        self.graph.open()

    def close(self):
        self.graph.close()

    @property
    def name(self) -> str:
        return self.graph.name
    
    @property
    def graph_series(self) -> dict[str, GraphSeries]:
        return {k:GraphSeries(x) for (k,x) in self.graph.graph_series.items()}



class GraphSeries:
    def __init__(self, graph_series) -> None:
        self.graph_series = graph_series

    def set_context(self, context: str):
        self.graph_series.set_context(context)
    
    @property
    def name(self) -> str:
        return self.graph_series.name



from .Analysis import Analysis
from .Application import Application
from .DataSet import DataSet
from .Design import Design
from .Equations import Equations
from .Graph import Graph
from .GraphSeries import GraphSeries
from .Param import Param
from .Part import Part
from .Variable import Variable
from .Workspace import Workspace